import whatap.net as net
from whatap.value.map_value import MapValue
from whatap.value.text_value import TextValue
from whatap.value.decimal_value import DecimalValue
from whatap.value.double_value import DoubleValue
from whatap.value.boolean_value import BooleanValue
import whatap.agent.conf.configure as configure
import whatap.osinfo.stack_dump as stack_dump
import whatap.util.logging_util as logging_util
import whatap.value as value
import threading, os, time
import whatap.agent.control.handle as handle
import whatap.ext.simplepluginWrapper as simpleplugin
start = False

def initControlHandler():
    global start
    if not start:
        start = True

        t = threading.Thread(target=runControl)
        t.setDaemon(True)
        t.start()

def runControl():
    while True:
        try:
            __runControl()
            time.sleep(0.001)
        except Exception, e:
            logging_util.logStack(e)
            time.sleep(0.1)

def __runControl():
    import whatap.net.receiver as receiver
    import whatap.agent.data.data_pack as data_pack
    import whatap.agent.conf.configure as conf
    import whatap.agent.secure.security_master as secu
    pcode = secu.GetSecurityMaster().pcode
    oid = secu.GetSecurityMaster().oid

    while True:
        #logging_util.debug("control handler trying to receive param pack") 
        p = receiver.getReceiver().get()
        #logging_util.debug("p.id:", p.id)
        #logging_util.debug( 'control_handler.runControl step -1')
        if p.id == net.PARAM_GET_ENV:
            r = p.toResponse()
            r.pcode = pcode
            r.oid = oid
            m = MapValue()
            for k, v in os.environ.items():
                m.putString(k,v)
            r.put('env', m)

            data_pack.SendSecure(r)
        elif p.id == net.PARAM_SET_CONFIG:
            #logging_util.debug('control_handler.runControl step -2')
            c = p.getMap("config")
            #logging_util.debug('control_handler.runControl step -3')
            if c:
                m = {}
                for k in c.keys():
                    m[k] = c.get(k).toObject()
                    #logging_util.debug('param_set_config {}={}'.format(k, m[k]))   
                #logging_util.debug('control_handler.runControl step -4')
                conf.SetValues(m)
            r = p.toResponse()
            r.pcode = pcode
            r.oid = oid
            data_pack.SendSecure(p)

        elif p.id == net.PARAM_CONFIGURE_GET:
            r = p.toResponse()
            r.pcode = pcode
            r.oid = oid

            def h2(k, v):
                r.put(k, TextValue(value=v))
            m = configure.GetConfig().populateAllKeyValues(h2)

            data_pack.SendSecure(p)

        elif p.id == net.PARAM_THREAD_LIST:

            r = p.toResponse()
            r.pcode = pcode
            r.oid = oid
            mv = stack_dump.getAll()

            p.put("thread_list", mv)
            data_pack.SendSecure(p)

        elif p.id == net.PARAM_THREAD_DETAIL:
            r = p.toResponse()
            r.pcode = pcode
            r.oid = oid
            threadid = p.get("thread_id").toObject()
            m = stack_dump.getThreadDetail(threadid)
            p.put(str(threadid), m)
            data_pack.SendSecure(p)
        elif p.id == net.PARAM_AGENT_LOG_LIST:
            r = p.toResponse()
            r.pcode = pcode
            r.oid = oid
            mv = MapValue()
            mv.put("default", DecimalValue(value=1024))
            p.put("files", mv)
            data_pack.SendSecure(p)
        elif p.id == net.PARAM_AGENT_LOG_READ:
            r = p.toResponse()
            r.pcode = pcode
            r.oid = oid
            p.put("before", DecimalValue(value=0))
            p.put("next", DecimalValue(value=-1))
            p.putString("text", logging_util.getAllLogs())
            data_pack.SendSecure(p)
        elif p.id == net.PARAM_REMOTE_CMD:
            cmd = p.getString("cmd")
            logging_util.debug("control handler remote cmd :{}".format(cmd))
            p = p.toResponse()
            p.pcode = pcode
            p.oid = oid
            if cmd == "list":
                m = handle.getCommandList()
                p.put("result", m)
            elif cmd == "execute" :
                scriptSlot = p.getString("script")
                options = p.get("args")
            
                m = handle.execute(scriptSlot, options)
                p.put("result", m)
                logging_util.debug("remote cmd scriptSlot:{} options:{}".format(scriptSlot, ",".join([options.getString(i) for i in range(options.size())]))) 
                                
            elif cmd == "getInput":
                inputfile = p.getString("inputFile")
                m = MapValue()
                p.put("result", m)
                try:
                    inputText = simpleplugin.getInput(inputfile)
                    m.putString("inputToml", inputText)
                    m.put("success", BooleanValue(True))
                except Exception, e:
                    logging_util.debug('control handler getInput error:{}'.format(e))
                    logging_util.debug(logging_util.getStackTrace())
                    m.putString("error", str(e))
                    m.put("success", BooleanValue(False))
                    
            elif cmd == "setInput":
                inputToml = p.getString("inputToml")
                inputfile = p.getString("inputFile")

                m = MapValue()
                p.put("result", m)
                try:
                    simpleplugin.setInput(inputfile, inputToml)
                    m.put("success", BooleanValue(True))
                except Exception, e:
                    logging_util.debug('control handler setInput error:{}'.format(e))
                    logging_util.debug(logging_util.getStackTrace())
                    m.putString("error", str(e))
                    m.put("success", BooleanValue(False))
                
            elif cmd == "getAllInput":
                m = MapValue()
                p.put("result", m)
                
                try:
                    inputs = simpleplugin.getAllInput()
                    allInputs = m.newList("inputs")
                    for input in inputs:
                        allInputs.addString(input)
                
                    m.put("success", BooleanValue(True))
                except Exception, e:
                    logging_util.debug("getAllInput error: {}".format(e))
                    logging_util.debug(logging_util.getStackTrace())
                    m.putString("error", str(e))
                    m.put("success", BooleanValue(False))
                
            elif cmd == "deleteInput":
                m = MapValue()
                p.put("result", m)
                inputfiles = p.get("inputFileList")
                for i in range(1,inputfiles.size()):
                    inputFile = inputfiles.getString(i)
                    try:
                        simpleplugin.deleteInput(inputFile)
                        m.put(inputFile, BooleanValue(True))
                    except Exception, e:
                        logging_util.debug("deleteInput error: {}".format(e))
                        m.put(inputFile, BooleanValue(False))
            data_pack.SendSecure(p)

