import time
import whatap.pack.tagcount_pack as tagcount_pack 
import whatap.osinfo as osinfo
import whatap.agent.secure.security_master as secure
from whatap.util.hash_util import HashUtil as hashutil
from whatap.value.list_value import ListValue 

class TaskInfraProcessTopN:
    PROCESS_TOPN = "infra_process_topn"

    PROCESS_TOPN_COMMAND = "cmd"
    PROCESS_TOPN_OWNER = "owner"
    PROCESS_TOPN_CMDLINE = "args"
    PROCESS_TOPN_STIME = "stime"
    PROCESS_TOPN_PID = "pid"
    PROCESS_TOPN_PPID = "ppid"
    PROCESS_TOPN_CPU = "cpu"
    PROCESS_TOPN_RSS = "rss"
    PROCESS_TOPN_VSZ = "vsz"
    PROCESS_TOPN_TOTAL_CHAR_IO = "totalCharIo"
    PROCESS_TOPN_THREAD_COUNT = "threadCnt"

    def __init__(self, interval = 5, topn = 20):
        self.intervalValue = interval
        self.recType = 2
        self.topn = topn

    def interval(self):
        return self.intervalValue

    def process(self, os_, pcode, oid, now):
        topn = osinfo.GetTopNProc(self.topn)
        if not topn:
            return None

        secu = secure.GetSecurityMaster()
        p = tagcount_pack.TagCountPack()
        p.pcode = pcode
        p.oid = oid
        p.time = now
        p.category = TaskInfraProcess.PROCESS_TOPN
        p.putTag("oname", str(secu.oname))
        p.putTag("!rectype", str(self.recType))

        
        idList = ListValue.ListValue()
        commandList = ListValue.ListValue()
        ownerList = ListValue.ListValue()
        cmdlineList = ListValue.ListValue()
        stimeList = ListValue.ListValue()
        pidList = ListValue.ListValue()
        ppidList = ListValue.ListValue()
        cpuList = ListValue.ListValue()
        rssList = ListValue.ListValue()
        vszList = Listvalue.ListValue()
        totalCharIoList = ListValue.ListValue()
        threadCountList = Listvalue.ListValue()

        for proc in topn:
            key = hashutil.hashFromString("{}{}{}{}{}{}").format(topn.cmd1, topn.user, topn.cmd2, topn.createTime, topn.pid, topn.ppid)
            idList.addString(key)
            commdList.addString(topn.cmd1)
            ownerList.addString(topn.user)
            cmdlineList.addString(topn.cmd2)
            stimeList.addLong(top.createTime)
            pidList.addLong(topn.pid)
            ppidList.addLong(topn.ppid)
            cpuList.addFloat(topn.cpu)
            rssList.addLong(topn.memoryBytes)
            vszList.addLong(topn.vsz)
            totalCharIoList.addLong(topn.totalCharIO)
            threadCountList.addLong(topn.threadCount)

        p.put("@id", idList)
        p.put(PROCESS_TOPN_COMMAND, commandList)
        p.put(PROCESS_TOPN_OWNER, ownerList)
        p.put(PROCESS_TOPN_CMDLINE, cmdlineList)
        p.put(PROCESS_TOPN_STIME, stimeList)
        p.put(PROCESS_TOPN_PID, pidList)
        p.put(PROCESS_TOPN_PPID, ppidList)
        p.put(PROCESS_TOPN_CPU, cpuList)
        p.put(PROCESS_TOPN_RSS, rssList)
        p.put(PROCESS_TOPN_VSZ, vszList)
        p.put(PROCESS_TOPN_TOTAL_CHAR_IO, totalCharIoList)
        p.put(PROCESS_TOPN_THREAD_COUNT, threadCountList)

        return p


