import platform
import os

_getcpumemory = None
_getcpunum = None
_getcpu = None
_getmemory = None
_getswapmemory = None
_getcpustat = None
_getcpucore = None
_getcputype = None
_getmemorysize = None
_getdisk = None
_getalldisk = None
_getdiskio = None
_initzfsinfo = None
_getzfsinfo = None
_initDisk = None
_endDisk = None
_getnet = None
_getallnet = None
_getproc = None
_getallproc = None
_gettopnandgroupproc = None
_getlogevent = None
_initlogevent = None
_getmachinearc = None
_getServerInfoV2 = None

processes = []
system = platform.system().lower()

class CPUStat:
    def __init__(self):
        self.runqueue = 0
        self.blockqueue = 0
        self.pendingqueue = 0
        self.waitqueue = 0
        self.forkCount = 0
        self.execCount = 0
        self.ctx = 0
        self.interrupt = 0
        self.syscall = 0


if 'linux' == system:
    import cpu_memory as cpu_memory
    _getcpumemory = cpu_memory.GetCpuMemory

    import system_info_linux as system_info
    _getcpunum = system_info.GetCPUNum
    _getcputype = system_info.GetCPUType
    _getmemorysize = system_info.GetMemorySize
    _getOSRelease = system_info.GetOSRelease
    import disk_linux
    _getdisk = disk_linux.getDisk
    import network_linux
    _getnet = network_linux.GetNicUtil
    import proc_linux
    _getproc = proc_linux.GetProc
    import log_event_linux
    _getlogevent = log_event_linux.getNewLogEvents
    _initlogevent = log_event_linux.initFileLogWatcher
    _getmachinearc = system_info.GetMachineArch
elif 'sunos' == system:
    import cpu_memory_sunos as cpu_memory
    _getcpumemory = cpu_memory.GetCpuMemory
    _getcpu = cpu_memory.GetCpu
    _getmemory = cpu_memory.GetMemory
    _getswapmemory = cpu_memory.GetSwapMemory
    _getcpustat = cpu_memory.GetCpuStat
    _getcpucore = cpu_memory.GetCpuCore
    import system_info_sunos as system_info
    import system_info_sunos
    _getcpunum = system_info_sunos.GetCPUNum
    _getcputype = system_info_sunos.GetCPUType
    _getmemorysize = system_info_sunos.GetMemorySize
    _getOSRelease = system_info_sunos.GetOSRelease
    import disk_sunos 
    _getdisk = disk_sunos.getDisk #fs
    _getalldisk = disk_sunos.getDiskIoInfo
    _getdiskio = disk_sunos.GetDiskIO
    _getzfsinfo = disk_sunos.GetZfsInfo
    _initzfsinfo = disk_sunos.initUpdateZfs
    import network_sunos
    _getnet = network_sunos.GetNicUtil
    _getallnet = network_sunos.GetNetInfo
    import proc_sunos
    _getproc = proc_sunos.GetProc
    _getallproc = proc_sunos.GetTotalProc
    _gettopnandgroupproc = proc_sunos.GetTopNAndGroupProc
    import log_event_linux
    _getlogevent = log_event_linux.getNewLogEvents
    _initlogevent = log_event_linux.initFileLogWatcher
    _getmachinearc = system_info_sunos.GetMachineArch
    _getServerInfoV2 = system_info_sunos.GetServerInfoV2
elif 'aix' == system: 
    os.environ['LANG'] = 'en_US'
    import cpu_memory_aix as cpu_memory
    _getcpumemory = cpu_memory.GetCpuMemory

    import system_info_aix 
    _getcpunum = system_info_aix.GetCPUNum
    _getcputype = system_info_aix.GetCPUType
    _getmemorysize = system_info_aix.GetMemorySize
    _getOSRelease = system_info_aix.GetOSRelease
    import disk_aix 
    _getdisk = disk_aix.getDisk
    import network_aix
    _getnet = network_aix.GetNicUtil
    import proc_aix
    _getproc = proc_aix.GetProc
    import log_event_linux
    _getlogevent = log_event_linux.getNewLogEvents
    _initlogevent = log_event_linux.initFileLogWatcher
    _getmachinearc = system_info_aix.GetMachineArch
    _getServerInfoV2 = system_info_aix.GetServerInfoV2
elif 'hp-ux' == system:
    import cpu_memory_hpux as cpu_memory
    _getcpumemory = cpu_memory.GetCpuMemory

    import system_info_hpux as system_info
    _getcpunum = system_info.GetCPUNum
    _getcputype = system_info.GetCPUType
    _getmemorysize = system_info.GetMemorySize
    _getOSRelease = system_info.GetOSRelease
    _getmachinearc = system_info.GetMachineArch
    import disk_hpux
    _getdisk = disk_hpux.getDisk
    import proc_hpux
    _getproc = proc_hpux.GetProc
    import network_hpux
    _getnet = network_hpux.GetNicUtil
    import log_event_linux
    _getlogevent = log_event_linux.getNewLogEvents
    _initlogevent = log_event_linux.initFileLogWatcher
    _getServerInfoV2 = system_info_hpux.GetServerInfoV2

def GetCpu():
    return _getcpu()

def GetMemory():
    return _getmemory()

def GetSwapMemory():
    return _getswapmemory()

def GetCpuStat():
    return _getcpustat()

def GetCpuCore():
    return _getcpucore()

def GetCpuMemory():
    return _getcpumemory()

def GetCPUNum():
    return _getcpunum()

def GetLCore():
    return system_info.serverInfoV2["lCores"]

def GetCPUType():
    return _getcputype()

def GetMemorySize():
    return _getmemorysize()

def GetUptime():
    import time
    return int(time.time())

def GetDisk():
    return _getdisk()

def GetTotalDisk():
    return _getalldisk()

def GetDiskIO():
    return _getdiskio()

def InitZfsInfo():
    if _initzfsinfo == None:
        return None
    return _initzfsinfo()

def GetZfsInfo():
    return _getzfsinfo()

def GetNicUtil():
    return _getnet()

def GetTotalNet():
    return _getallnet()

def GetProc():
    return _getproc()

def GetTotalProc():
    return _getallproc()

def GetTopNAndGroupProc(n):
    return _gettopnandgroupproc(n)

def GetLogEvents():
    return _getlogevent()

def InitLogEvent():
    _initlogevent()

def GetMachineArch():
    return _getmachinearc()

import tcp_check

def TCPCheckSchedule(checkurl, timeout, onPerf):
    return tcp_check.tcpCheckSchedule(checkurl, timeout, onPerf)

def GetOSRelease():
    return _getOSRelease()

def GetServerInfoV2():
    return _getServerInfoV2()

