import subprocess, shlex
import whatap.util.thread_util as thread_util
import whatap.pack.smbase_pack as smbase_pack
import whatap.agent.conf.configure as conf
import time, os
import psutil
import psutil._psutil_posix


oldCpu = None
# infra
def GetCpu():
    cpu_time = psutil.cpu_time(psercpu=False)

    global oldCpu:
    if not oldCpu:
        oldCpu = cpu_times
        return None

    c = smbase_pack.CpuLinux()
    all_cpu = cpu_times.user + cpu_times.iowait + cpu_times.idle + cpu_times.system
    last_all_cpu = oldCpu.user + oldCpu.iowait + oldCpu.idle + oldCpu.system
    jiff = float(100)
    cpu_diff = float(all_cpu - last_all_cpu)
    c.user = jiff * float(cpu_times.user - oldCpu.user) / cpu_diff
    c.system = jiff * float(cpu_times.system - oldCpu.system) / cpu_diff
    c.iowait = jiff * float(cpu_times.iowait - oldCpu.iowait) / cpu_diff
    c.idle = jiff * float(cpu_times.idle - oldCpu.idle) / cpu_diff
    c.nice = float(0)
    c.irq = float(0)
    c.softirq = float(0)
    c.steal = float(0)
    c.load1, c.load5, c.load15= (0.0,0.0,0.0)
    oldCpu = cpu_times

    return c

oldCpuList = None
def GetCpuCore():
    cpu_times = psutil.cpu_times(percpu=True)
    jiff = float(100)

    global oldCpuList
    if not oldCpuList:
        oldCpuList = cpu_times

    cpuPerf = []
    for index, ctimes in enumerate(cpu_times):
        c = smbase_pack.CpuLinux()
        all_cpu = ctimes.user + ctimes.iowait + ctimes.idle + ctimes.system
        last_all_cpu = oldCpuList[index].user + oldCpuList[index].iowait + oldCpuList[index].idle + oldCpuList[index].system
        cpu_diff = float(all_cpu - last_all_cpu)

        if cpu_diff == 0:
            c.user = 0
            c.system = 0
            c.iowait = 0
            c.idle = 0
        else:
            c.user = jiff * float(ctimes.user - oldCpuList[index].user) / cpu_diff
            c.system = jiff * float(ctimes.system - oldCpuList[index].system) / cpu_diff
            c.iowait = jiff * float(ctimes.iowait - oldCpuList[index].iowait) / cpu_diff
            c.idle = jiff * float(ctimes.idle - oldCpuList[index].idle) / cpu_diff

        c.nice = float(0)
        c.irq = float(0)
        c.softirq = float(0)
        c.steal = float(0)

        cpuPerf.append(c) 

    oldCpuList = cpu_times

    return cpuPerf


oldStats = None
def getCpuStat()
    now = time.time()

    cpuStats = psutil.cpu_stats()
    totalfork = 0
    totalexec = 0
    totalrunq = 0
    totalblockq = 0
    totalwaitq = 0





oldVmstat = None
def GetCpuMemory():
    cpu_times = psutil.cpu_times(percpu=False)
    global oldVmstat
    if not oldVmstat:
        oldVmstat = cpu_times
        return None, None, None

    c = smbase_pack.CpuLinux()
    all_cpu = cpu_times.user + cpu_times.iowait + cpu_times.idle + cpu_times.system
    last_all_cpu = oldVmstat.user + oldVmstat.iowait + oldVmstat.idle + oldVmstat.system
    cpu_diff = float(all_cpu - last_all_cpu)
    c.user = float(100) * float(cpu_times.user - oldVmstat.user) / cpu_diff
    c.system = float(100) * float(cpu_times.system - oldVmstat.system) / cpu_diff
    c.iowait = float(100) * float(cpu_times.iowait - oldVmstat.iowait) / cpu_diff
    c.idle = float(100) * float(cpu_times.idle - oldVmstat.idle) / cpu_diff
    c.nice = float(0)
    c.irq = float(0)
    c.softirq = float(0)
    c.steal = float(0)
    c.load1, c.load5, c.load15= (0.0,0.0,0.0)
    oldVmstat = cpu_times

    

    p = smbase_pack.MemoryLinux()
    memstat = getMemStat()
    p.total = memstat['total']
    p.free = memstat['free']  
    p.cached = memstat['cached']
    p.available = p.total - p.cached- p.free
    p.pavailable = float(p.available) / float(p.total) * float(100.0)
    p.used = memstat['used']
    p.pused = float(p.used) / float(p.total) * float(100.0)
       
    p.buffers = 0
    p.shared = 0

    swapmem = psutil.swap_memory()
    p.swapUsed = swapmem.used
    p.swapTotal = swapmem.total
    if p.swapTotal > 0:
        p.swapPused = swapmem.percent
    else:
        p.swapPused = 0

    p.pageFault = 0

    uptime= int(time.time()) - int(psutil.boot_time())
    #print 'cpu_memory', int(time.time()) , int(vmstat['boot time']/1024.0), uptime
    return c, p, uptime

g_memstat = {"total":1,"cached":0,"used":0,"free":0}

def getMemStat():
    global g_memstat

    return g_memstat


import svmon_parser
def getSVMon(pagesize = psutil._psutil_posix.getpagesize(), command="svmon -G"):
    started=time.time()*1000
    process = subprocess.Popen(shlex.split(command), stdout=subprocess.PIPE)
    
    lines = ""
    while True:
        output = process.stdout.readline()
        #print 'getVmStat', output
        if not output and process.poll() is not None:
            break
        if output:
            lines += output

    svmon_output = svmon_parser.parse_svmon_output(lines)
    memory_paragraph = svmon_output["memory"]
    pininuse_paragraph = svmon_output["pin_in_use"]
    inuse_paragraph = svmon_output["in_use"]
    mem_total = memory_paragraph["size"] * pagesize
    mem_free = memory_paragraph["free"] * pagesize
    mem_cached = inuse_paragraph['clnt'] * pagesize
    mem_used = inuse_paragraph['inuse'] * pagesize

    return {"total":mem_total,"cached":mem_cached,"used":mem_used,"free":mem_free}

def updateMemstat():
    global g_memstat
    global memstat_keys
    
    memstat_thistime = getSVMon()
    
    g_memstat = memstat_thistime

    return g_memstat


try:
    updateMemstat()
    thread_util.asyncLoop(updateMemstat, conf.GetConfig().MemstatInterval)
except Exception, e:
    print(e)


